@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 409703 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 413587 $")
 
 #include "asterisk/module.h"
 #include "asterisk/http.h"
@@ -79,6 +79,7 @@ struct ast_websocket {
 	size_t reconstruct;               /*!< Number of bytes before a reconstructed payload will be returned and a new one started */
 	unsigned int secure:1;            /*!< Bit to indicate that the transport is secure */
 	unsigned int closing:1;           /*!< Bit to indicate that the session is in the process of being closed */
+	unsigned int close_sent:1;        /*!< Bit to indicate that the session close opcode has been sent and no further data will be sent */
 };
 
 /*! \brief Structure definition for protocols */
@@ -120,6 +121,8 @@ static void session_destroy_fn(void *obj)
 {
 	struct ast_websocket *session = obj;
 
+	ast_websocket_close(session, 0);
+
 	if (session->f) {
 		fclose(session->f);
 		ast_verb(2, "WebSocket connection from '%s' closed\n", ast_sockaddr_stringify(&session->address));
@@ -188,6 +191,11 @@ int AST_OPTIONAL_API_NAME(ast_websocket_remove_protocol)(const char *name, ast_w
 int AST_OPTIONAL_API_NAME(ast_websocket_close)(struct ast_websocket *session, uint16_t reason)
 {
 	char frame[4] = { 0, }; /* The header is 2 bytes and the reason code takes up another 2 bytes */
+	int res;
+
+	if (session->close_sent) {
+		return 0;
+	}
 
 	frame[0] = AST_WEBSOCKET_OPCODE_CLOSE | 0x80;
 	frame[1] = 2; /* The reason code is always 2 bytes */
@@ -196,8 +204,12 @@ int AST_OPTIONAL_API_NAME(ast_websocket_close)(struct ast_websocket *session, ui
 	put_unaligned_uint16(&frame[2], htons(reason ? reason : 1000));
 
 	session->closing = 1;
+	session->close_sent = 1;
 
-	return (fwrite(frame, 1, 4, session->f) == 4) ? 0 : -1;
+	ao2_lock(session);
+	res = (fwrite(frame, 1, 4, session->f) == 4) ? 0 : -1;
+	ao2_unlock(session);
+	return res;
 }
 
 
@@ -233,14 +245,23 @@ int AST_OPTIONAL_API_NAME(ast_websocket_write)(struct ast_websocket *session, en
 		put_unaligned_uint64(&frame[2], htonl(actual_length));
 	}
 
+	ao2_lock(session);
+	if (session->closing) {
+		ao2_unlock(session);
+		return -1;
+	}
+
 	if (fwrite(frame, 1, header_size, session->f) != header_size) {
+		ao2_unlock(session);
 		return -1;
 	}
 
 	if (fwrite(payload, 1, actual_length, session->f) != actual_length) {
+		ao2_unlock(session);
 		return -1;
 	}
 	fflush(session->f);
+	ao2_unlock(session);
 
 	return 0;
 }
@@ -414,7 +435,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 		*payload = &buf[frame_size]; /* payload will start here, at the end of the options, if any */
 		frame_size = frame_size + (*payload_len); /* final frame size is header + optional headers + payload data */
 		if (frame_size > MAXIMUM_FRAME_SIZE) {
-			ast_log(LOG_WARNING, "Cannot fit huge websocket frame of %zd bytes\n", frame_size);
+			ast_log(LOG_WARNING, "Cannot fit huge websocket frame of %zu bytes\n", frame_size);
 			/* The frame won't fit :-( */
 			ast_websocket_close(session, 1009);
 			return -1;
@@ -433,7 +454,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 		}
 
 		if (!(new_payload = ast_realloc(session->payload, (session->payload_len + *payload_len)))) {
-			ast_log(LOG_WARNING, "Failed allocation: %p, %zd, %"PRIu64"\n",
+			ast_log(LOG_WARNING, "Failed allocation: %p, %zu, %"PRIu64"\n",
 				session->payload, session->payload_len, *payload_len);
 			*payload_len = 0;
 			ast_websocket_close(session, 1009);
@@ -483,15 +504,9 @@ int AST_OPTIONAL_API_NAME(ast_websocket_read)(struct ast_websocket *session, cha
 			frame_size += (*payload_len);
 		}
 
-		if (!session->closing) {
-			ast_websocket_close(session, 0);
-		}
-
-		fclose(session->f);
-		session->f = NULL;
-		ast_verb(2, "WebSocket connection from '%s' closed\n", ast_sockaddr_stringify(&session->address));
+		session->closing = 1;
 	} else {
-		ast_log(LOG_WARNING, "WebSocket unknown opcode %d\n", *opcode);
+		ast_log(LOG_WARNING, "WebSocket unknown opcode %u\n", *opcode);
 		/* We received an opcode that we don't understand, the RFC states that 1003 is for a type of data that can't be accepted... opcodes
 		 * fit that, I think. */
 		ast_websocket_close(session, 1003);
@@ -698,7 +713,7 @@ static void websocket_echo_callback(struct ast_websocket *session, struct ast_va
 		} else if (opcode == AST_WEBSOCKET_OPCODE_CLOSE) {
 			break;
 		} else {
-			ast_debug(1, "Ignored WebSocket opcode %d\n", opcode);
+			ast_debug(1, "Ignored WebSocket opcode %u\n", opcode);
 		}
 	}
 
